import { isCallExpression, isExportDeclaration, isImportDeclaration, isStringLiteral, SyntaxKind, visitEachChild } from 'typescript';
import { relative, } from 'path';
/**
 * @internal
 */
export var importQueryParam = '__mi';
/**
 * @internal
 */
export var importContextQueryParam = '__mi_ctx';
/**
 * Create source code transformer
 * @internal
 * @param cwd - working dir
 * @param resolve - dependencies resolver
 * @param file - target file
 * @param preloadList - list modules to preload
 */
export function createImportsTransformer(cwd, resolve, file, preloadList) {
    /**
     * Creates import url.
     * @param target - for the dependency
     */
    var url = function (target) {
        var url;
        var resolution = resolve(file, target);
        if (resolution.isExternal) {
            url = resolution.resolution;
        }
        else if (resolution.isUrl) {
            url = resolution.resolution;
        }
        else {
            var resolved = relative(cwd, resolution.resolution);
            resolved = resolved.replace(/\\/g, '/');
            var context_1 = '';
            while (resolved.startsWith("../")) {
                context_1 += "../";
                resolved = resolved.substr(3);
            }
            url = "/" + resolved + "?" + importQueryParam + "=true" + (context_1 ? "&" + importContextQueryParam + "=" + context_1 : '');
        }
        if (preloadList.indexOf(url) < 0) {
            preloadList.push(url);
        }
        return url;
    };
    return function (context) {
        // creates visitor
        var visitor = function (node) {
            var _a;
            // dynamic import call
            if (isCallExpression(node) && node.expression.kind === SyntaxKind.ImportKeyword) {
                var arg = node.arguments[0];
                if (isStringLiteral(arg)) {
                    return context.factory.updateCallExpression(node, node.expression, node.typeArguments, [
                        context.factory.createStringLiteral(url(arg.text))
                    ]);
                }
                // todo report problematic import
            }
            // export ... from ...
            if (isExportDeclaration(node) && !node.isTypeOnly) {
                var moduleSpecifier = node.moduleSpecifier;
                if (moduleSpecifier && isStringLiteral(moduleSpecifier)) {
                    return context.factory.updateExportDeclaration(node, node.decorators, node.modifiers, node.isTypeOnly, node.exportClause, context.factory.createStringLiteral(url(moduleSpecifier.text)));
                }
            }
            // import ... from ...
            if (isImportDeclaration(node) && !((_a = node.importClause) === null || _a === void 0 ? void 0 : _a.isTypeOnly)) {
                var moduleSpecifier = node.moduleSpecifier;
                if (isStringLiteral(moduleSpecifier)) {
                    return context.factory.updateImportDeclaration(node, node.decorators, node.modifiers, node.importClause, context.factory.createStringLiteral(url(moduleSpecifier.text)));
                }
                return node;
            }
            return visitEachChild(node, visitor, context);
        };
        return function (node) {
            return visitEachChild(node, visitor, context);
        };
    };
}
//# sourceMappingURL=create-imports-transformer.js.map