import { __awaiter, __generator } from "tslib";
import fs from 'fs';
import { join } from 'path';
import { promisify } from 'util';
var readStats = promisify(fs.stat);
/**
 * Create source code provider.
 * @internal
 * @param cwd - working directory
 * @param transpile - typescript/es transpile function
 * @param options - transpile options
 */
export var createSourceCodeProvider = function (cwd, options, transpile) {
    return function (file) { return __awaiter(void 0, void 0, void 0, function () {
        var cachedValue, stats;
        var _a;
        return __generator(this, function (_b) {
            switch (_b.label) {
                case 0:
                    file = join(cwd, file);
                    if (!(!options.disabled && options.provider)) return [3 /*break*/, 5];
                    return [4 /*yield*/, options.provider.get(file)];
                case 1:
                    cachedValue = _b.sent();
                    return [4 /*yield*/, readStats(file)];
                case 2:
                    stats = _b.sent();
                    if (!(!cachedValue || cachedValue.timestamp < stats.mtimeMs)) return [3 /*break*/, 4];
                    _a = { timestamp: stats.mtimeMs };
                    return [4 /*yield*/, transpile(file)];
                case 3:
                    cachedValue = (_a.source = _b.sent(), _a);
                    options.provider.set(file, cachedValue);
                    _b.label = 4;
                case 4: return [2 /*return*/, cachedValue.source];
                case 5: return [2 /*return*/, transpile(file)];
            }
        });
    }); };
};
//# sourceMappingURL=create-source-code-provider.js.map