import { __awaiter, __generator, __spreadArray } from "tslib";
import fs from 'fs';
import { basename } from 'path';
import { transpileModule } from 'typescript';
import { promisify } from 'util';
import { createImportsTransformer } from './create-imports-transformer';
var readFile = promisify(fs.readFile);
/**
 * Create source code transpiler.
 * @internal
 * @param cwd - working dir
 * @param compilerOptions - typescript compiler options
 * @param transformers - list of custom transformers
 * @param resolve - dependencies resolver
 * @param preloadList - list of modules to preload
 */
export var createTranspiler = function (cwd, compilerOptions, transformers, resolve, preloadList) {
    return function (file) { return __awaiter(void 0, void 0, void 0, function () {
        var source, result;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0: return [4 /*yield*/, readFile(file)];
                case 1:
                    source = (_a.sent()).toString('utf8');
                    result = transpileModule(source, {
                        fileName: basename(file),
                        compilerOptions: compilerOptions,
                        transformers: {
                            after: __spreadArray(__spreadArray([], transformers), [
                                createImportsTransformer(cwd, resolve, file, preloadList)
                            ])
                        }
                    });
                    return [2 /*return*/, result.outputText];
            }
        });
    }); };
};
//# sourceMappingURL=create-transpiler.js.map